"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import publicendpoint as components_publicendpoint
from openrouter.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class ListEndpointsZdrResponseTypedDict(TypedDict):
    r"""Returns a list of endpoints"""

    data: List[components_publicendpoint.PublicEndpointTypedDict]


class ListEndpointsZdrResponse(BaseModel):
    r"""Returns a list of endpoints"""

    data: List[components_publicendpoint.PublicEndpoint]



"""Generate OAuth2 authorization URL"""

from dataclasses import dataclass
from typing import TYPE_CHECKING, Literal, Optional, Union
from urllib.parse import ParseResult, urlencode

if TYPE_CHECKING:
    from openrouter.sdk import OpenRouter


@dataclass
class CreateAuthorizationUrlRequestBase:
    """Base request parameters for creating an authorization URL"""
    callback_url: Union[str, ParseResult]
    limit: Optional[float] = None


@dataclass
class CreateAuthorizationUrlRequestWithPKCE:
    """Request parameters with PKCE for creating an authorization URL"""
    callback_url: Union[str, ParseResult]
    code_challenge_method: Literal["S256", "plain"]
    code_challenge: str
    limit: Optional[float] = None


# Union type for request - either with PKCE or without
CreateAuthorizationUrlRequest = Union[
    CreateAuthorizationUrlRequestWithPKCE,
    CreateAuthorizationUrlRequestBase,
]


def _get_server_url(client: "OpenRouter") -> str:
    """
    Get the server URL from the client configuration

    Args:
        client: OpenRouter client instance

    Returns:
        The server URL

    Raises:
        ValueError: If no server URL is configured
    """
    server_url, _ = client.sdk_configuration.get_server_details()
    if not server_url:
        raise ValueError("No server URL configured")
    return server_url


def oauth_create_authorization_url(
    client: "OpenRouter",
    params: CreateAuthorizationUrlRequest,
) -> str:
    """
    Generate an OAuth2 authorization URL

    Generates a URL to redirect users to for authorizing your application. The
    URL includes the provided callback URL and, if applicable, the code
    challenge parameters for PKCE.

    Args:
        client: OpenRouter client instance
        params: Request parameters including callback URL and optional PKCE parameters

    Returns:
        The authorization URL as a string

    Raises:
        ValueError: If no server URL is configured or parameters are invalid

    See Also:
        - https://openrouter.ai/docs/use-cases/oauth-pkce
    """
    base_url = _get_server_url(client)

    # Build the auth URL
    auth_url = f"{base_url}/auth"

    # Build query parameters
    query_params = {
        "callback_url": str(params.callback_url),
    }

    # Add PKCE parameters if present
    if isinstance(params, CreateAuthorizationUrlRequestWithPKCE):
        query_params["code_challenge"] = params.code_challenge
        query_params["code_challenge_method"] = params.code_challenge_method

    # Add limit if present
    if params.limit is not None:
        query_params["limit"] = str(params.limit)

    # Construct final URL with query parameters
    return f"{auth_url}?{urlencode(query_params)}"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


OpenAIResponsesIncludable = Union[
    Literal[
        "file_search_call.results",
        "message.input_image.image_url",
        "computer_call_output.output.image_url",
        "reasoning.encrypted_content",
        "code_interpreter_call.outputs",
    ],
    UnrecognizedStr,
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import eventstreaming, get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Dict, List, Literal, Mapping, Optional, Union, overload


class SendAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Responses(BaseSDK):
    r"""beta.responses endpoints"""

    @overload
    def send(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Union[Literal[False], None] = None,
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.OpenResponsesNonStreamingResponse:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    def send(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Literal[True],
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStream[components.OpenResponsesStreamEvent]:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    def send(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Optional[bool] = False,
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateResponsesResponse:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.OpenResponsesRequest(
            input=utils.get_pydantic_model(
                input, Optional[components.OpenResponsesInput]
            ),
            instructions=instructions,
            metadata=metadata,
            tools=utils.get_pydantic_model(
                tools, Optional[List[components.OpenResponsesRequestToolUnion]]
            ),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[components.OpenAIResponsesToolChoiceUnion]
            ),
            parallel_tool_calls=parallel_tool_calls,
            model=model,
            models=models,
            text=utils.get_pydantic_model(
                text, Optional[components.OpenResponsesResponseText]
            ),
            reasoning=utils.get_pydantic_model(
                reasoning, OptionalNullable[components.OpenResponsesReasoningConfig]
            ),
            max_output_tokens=max_output_tokens,
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            prompt_cache_key=prompt_cache_key,
            previous_response_id=previous_response_id,
            prompt=utils.get_pydantic_model(
                prompt, OptionalNullable[components.OpenAIResponsesPrompt]
            ),
            include=include,
            background=background,
            safety_identifier=safety_identifier,
            store=store,
            service_tier=service_tier,
            truncation=truncation,
            stream=stream,
            provider=utils.get_pydantic_model(
                provider, OptionalNullable[components.Provider]
            ),
            plugins=utils.get_pydantic_model(
                plugins, Optional[List[components.Plugin]]
            ),
            user=user,
        )

        req = self._build_request(
            method="POST",
            path="/responses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.OpenResponsesRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createResponses",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "408",
                "413",
                "422",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            return unmarshal_json_response(
                components.OpenResponsesNonStreamingResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStream(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, operations.CreateResponsesResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res, http_res_text
            )
            raise errors.BadRequestResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res, http_res_text
            )
            raise errors.UnauthorizedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "402", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res, http_res_text
            )
            raise errors.PaymentRequiredResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "404", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res, http_res_text
            )
            raise errors.NotFoundResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "408", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.RequestTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.RequestTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "413", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.PayloadTooLargeResponseErrorData, http_res, http_res_text
            )
            raise errors.PayloadTooLargeResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityResponseErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res, http_res_text
            )
            raise errors.TooManyRequestsResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res, http_res_text
            )
            raise errors.InternalServerResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "502", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res, http_res_text
            )
            raise errors.BadGatewayResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "524", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.EdgeNetworkTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "529", "application/json"):
            http_res_text = utils.stream_to_text(http_res)
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res, http_res_text
            )
            raise errors.ProviderOverloadedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = utils.stream_to_text(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )

    @overload
    async def send_async(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Union[Literal[False], None] = None,
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> components.OpenResponsesNonStreamingResponse:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    @overload
    async def send_async(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Literal[True],
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> eventstreaming.EventStreamAsync[components.OpenResponsesStreamEvent]:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """

    async def send_async(
        self,
        *,
        input: Optional[
            Union[components.OpenResponsesInput, components.OpenResponsesInputTypedDict]
        ] = None,
        instructions: OptionalNullable[str] = UNSET,
        metadata: OptionalNullable[Dict[str, str]] = UNSET,
        tools: Optional[
            Union[
                List[components.OpenResponsesRequestToolUnion],
                List[components.OpenResponsesRequestToolUnionTypedDict],
            ]
        ] = None,
        tool_choice: Optional[
            Union[
                components.OpenAIResponsesToolChoiceUnion,
                components.OpenAIResponsesToolChoiceUnionTypedDict,
            ]
        ] = None,
        parallel_tool_calls: OptionalNullable[bool] = UNSET,
        model: Optional[str] = None,
        models: Optional[List[str]] = None,
        text: Optional[
            Union[
                components.OpenResponsesResponseText,
                components.OpenResponsesResponseTextTypedDict,
            ]
        ] = None,
        reasoning: OptionalNullable[
            Union[
                components.OpenResponsesReasoningConfig,
                components.OpenResponsesReasoningConfigTypedDict,
            ]
        ] = UNSET,
        max_output_tokens: OptionalNullable[float] = UNSET,
        temperature: OptionalNullable[float] = UNSET,
        top_p: OptionalNullable[float] = UNSET,
        top_k: Optional[float] = None,
        prompt_cache_key: OptionalNullable[str] = UNSET,
        previous_response_id: OptionalNullable[str] = UNSET,
        prompt: OptionalNullable[
            Union[
                components.OpenAIResponsesPrompt,
                components.OpenAIResponsesPromptTypedDict,
            ]
        ] = UNSET,
        include: OptionalNullable[List[components.OpenAIResponsesIncludable]] = UNSET,
        background: OptionalNullable[bool] = UNSET,
        safety_identifier: OptionalNullable[str] = UNSET,
        store: OptionalNullable[bool] = UNSET,
        service_tier: OptionalNullable[components.ServiceTier] = UNSET,
        truncation: OptionalNullable[components.Truncation] = UNSET,
        stream: Optional[bool] = False,
        provider: OptionalNullable[
            Union[components.Provider, components.ProviderTypedDict]
        ] = UNSET,
        plugins: Optional[
            Union[List[components.Plugin], List[components.PluginTypedDict]]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateResponsesResponse:
        r"""Create a response

        Creates a streaming or non-streaming response using OpenResponses API format

        :param input: Input for a response request - can be a string or array of items
        :param instructions:
        :param metadata: Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed.
        :param tools:
        :param tool_choice:
        :param parallel_tool_calls:
        :param model:
        :param models:
        :param text: Text output configuration including format and verbosity
        :param reasoning: Configuration for reasoning mode in the response
        :param max_output_tokens:
        :param temperature:
        :param top_p:
        :param top_k:
        :param prompt_cache_key:
        :param previous_response_id:
        :param prompt:
        :param include:
        :param background:
        :param safety_identifier:
        :param store:
        :param service_tier:
        :param truncation:
        :param stream:
        :param provider: When multiple model providers are available, optionally indicate your routing preference.
        :param plugins: Plugins you want to enable for this request, including their settings.
        :param user: A unique identifier representing your end-user, which helps distinguish between different users of your app. This allows your app to identify specific users in case of abuse reports, preventing your entire app from being affected by the actions of individual users. Maximum of 128 characters.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = components.OpenResponsesRequest(
            input=utils.get_pydantic_model(
                input, Optional[components.OpenResponsesInput]
            ),
            instructions=instructions,
            metadata=metadata,
            tools=utils.get_pydantic_model(
                tools, Optional[List[components.OpenResponsesRequestToolUnion]]
            ),
            tool_choice=utils.get_pydantic_model(
                tool_choice, Optional[components.OpenAIResponsesToolChoiceUnion]
            ),
            parallel_tool_calls=parallel_tool_calls,
            model=model,
            models=models,
            text=utils.get_pydantic_model(
                text, Optional[components.OpenResponsesResponseText]
            ),
            reasoning=utils.get_pydantic_model(
                reasoning, OptionalNullable[components.OpenResponsesReasoningConfig]
            ),
            max_output_tokens=max_output_tokens,
            temperature=temperature,
            top_p=top_p,
            top_k=top_k,
            prompt_cache_key=prompt_cache_key,
            previous_response_id=previous_response_id,
            prompt=utils.get_pydantic_model(
                prompt, OptionalNullable[components.OpenAIResponsesPrompt]
            ),
            include=include,
            background=background,
            safety_identifier=safety_identifier,
            store=store,
            service_tier=service_tier,
            truncation=truncation,
            stream=stream,
            provider=utils.get_pydantic_model(
                provider, OptionalNullable[components.Provider]
            ),
            plugins=utils.get_pydantic_model(
                plugins, Optional[List[components.Plugin]]
            ),
            user=user,
        )

        req = self._build_request_async(
            method="POST",
            path="/responses",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="text/event-stream" if stream else "application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", components.OpenResponsesRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createResponses",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "408",
                "413",
                "422",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            stream=True,
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            return unmarshal_json_response(
                components.OpenResponsesNonStreamingResponse, http_res, http_res_text
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return eventstreaming.EventStreamAsync(
                http_res,
                lambda raw: utils.unmarshal_json(
                    raw, operations.CreateResponsesResponseBody
                ).data,
                sentinel="[DONE]",
                client_ref=self,
            )
        if utils.match_response(http_res, "400", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res, http_res_text
            )
            raise errors.BadRequestResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "401", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res, http_res_text
            )
            raise errors.UnauthorizedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "402", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res, http_res_text
            )
            raise errors.PaymentRequiredResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "404", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res, http_res_text
            )
            raise errors.NotFoundResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "408", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.RequestTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.RequestTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "413", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.PayloadTooLargeResponseErrorData, http_res, http_res_text
            )
            raise errors.PayloadTooLargeResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "422", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.UnprocessableEntityResponseErrorData, http_res, http_res_text
            )
            raise errors.UnprocessableEntityResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "429", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res, http_res_text
            )
            raise errors.TooManyRequestsResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "500", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res, http_res_text
            )
            raise errors.InternalServerResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "502", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res, http_res_text
            )
            raise errors.BadGatewayResponseError(response_data, http_res, http_res_text)
        if utils.match_response(http_res, "503", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res, http_res_text
            )
            raise errors.ServiceUnavailableResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "524", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res, http_res_text
            )
            raise errors.EdgeNetworkTimeoutResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "529", "application/json"):
            http_res_text = await utils.stream_to_text_async(http_res)
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res, http_res_text
            )
            raise errors.ProviderOverloadedResponseError(
                response_data, http_res, http_res_text
            )
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        http_res_text = await utils.stream_to_text_async(http_res)
        raise errors.OpenRouterDefaultError(
            "Unexpected response received", http_res, http_res_text
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ResponseFormatTextGrammarTypedDict(TypedDict):
    grammar: str
    type: Literal["grammar"]


class ResponseFormatTextGrammar(BaseModel):
    grammar: str

    TYPE: Annotated[
        Annotated[Literal["grammar"], AfterValidator(validate_const("grammar"))],
        pydantic.Field(alias="type"),
    ] = "grammar"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.components import (
    openresponsesnonstreamingresponse as components_openresponsesnonstreamingresponse,
    openresponsesstreamevent as components_openresponsesstreamevent,
)
from openrouter.types import BaseModel
from openrouter.utils import eventstreaming
from typing import Union
from typing_extensions import TypeAliasType, TypedDict


class CreateResponsesResponseBodyTypedDict(TypedDict):
    r"""Successful response"""

    data: components_openresponsesstreamevent.OpenResponsesStreamEventTypedDict
    r"""Union of all possible event types emitted during response streaming"""


class CreateResponsesResponseBody(BaseModel):
    r"""Successful response"""

    data: components_openresponsesstreamevent.OpenResponsesStreamEvent
    r"""Union of all possible event types emitted during response streaming"""


CreateResponsesResponseTypedDict = TypeAliasType(
    "CreateResponsesResponseTypedDict",
    Union[
        components_openresponsesnonstreamingresponse.OpenResponsesNonStreamingResponseTypedDict,
        Union[
            eventstreaming.EventStream[
                components_openresponsesstreamevent.OpenResponsesStreamEventTypedDict
            ],
            eventstreaming.EventStreamAsync[
                components_openresponsesstreamevent.OpenResponsesStreamEventTypedDict
            ],
        ],
    ],
)


CreateResponsesResponse = TypeAliasType(
    "CreateResponsesResponse",
    Union[
        components_openresponsesnonstreamingresponse.OpenResponsesNonStreamingResponse,
        Union[
            eventstreaming.EventStream[
                components_openresponsesstreamevent.OpenResponsesStreamEvent
            ],
            eventstreaming.EventStreamAsync[
                components_openresponsesstreamevent.OpenResponsesStreamEvent
            ],
        ],
    ],
)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from enum import Enum
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional, Union


class GenerateAcceptEnum(str, Enum):
    APPLICATION_JSON = "application/json"
    TEXT_EVENT_STREAM = "text/event-stream"


class Embeddings(BaseSDK):
    r"""Text embedding endpoints"""

    def generate(
        self,
        *,
        input: Union[operations.Input, operations.InputTypedDict],
        model: str,
        provider: Optional[
            Union[
                operations.CreateEmbeddingsProvider,
                operations.CreateEmbeddingsProviderTypedDict,
            ]
        ] = None,
        encoding_format: Optional[
            Union[operations.EncodingFormat, operations.EncodingFormatTypedDict]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GenerateAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateEmbeddingsResponse:
        r"""Submit an embedding request

        Submits an embedding request to the embeddings router

        :param input:
        :param model:
        :param provider:
        :param encoding_format:
        :param user:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateEmbeddingsRequest(
            input=input,
            model=model,
            provider=utils.get_pydantic_model(
                provider, Optional[operations.CreateEmbeddingsProvider]
            ),
            encoding_format=encoding_format,
            user=user,
        )

        req = self._build_request(
            method="POST",
            path="/embeddings",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, text/event-stream;q=0",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEmbeddingsRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEmbeddings",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.CreateEmbeddingsResponseBody, http_res
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return http_res.text
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res
            )
            raise errors.PaymentRequiredResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res
            )
            raise errors.BadGatewayResponseError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res
            )
            raise errors.ServiceUnavailableResponseError(response_data, http_res)
        if utils.match_response(http_res, "524", "application/json"):
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res
            )
            raise errors.EdgeNetworkTimeoutResponseError(response_data, http_res)
        if utils.match_response(http_res, "529", "application/json"):
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res
            )
            raise errors.ProviderOverloadedResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def generate_async(
        self,
        *,
        input: Union[operations.Input, operations.InputTypedDict],
        model: str,
        provider: Optional[
            Union[
                operations.CreateEmbeddingsProvider,
                operations.CreateEmbeddingsProviderTypedDict,
            ]
        ] = None,
        encoding_format: Optional[
            Union[operations.EncodingFormat, operations.EncodingFormatTypedDict]
        ] = None,
        user: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        accept_header_override: Optional[GenerateAcceptEnum] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateEmbeddingsResponse:
        r"""Submit an embedding request

        Submits an embedding request to the embeddings router

        :param input:
        :param model:
        :param provider:
        :param encoding_format:
        :param user:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param accept_header_override: Override the default accept header for this method
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateEmbeddingsRequest(
            input=input,
            model=model,
            provider=utils.get_pydantic_model(
                provider, Optional[operations.CreateEmbeddingsProvider]
            ),
            encoding_format=encoding_format,
            user=user,
        )

        req = self._build_request_async(
            method="POST",
            path="/embeddings",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value=accept_header_override.value
            if accept_header_override is not None
            else "application/json;q=1, text/event-stream;q=0",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateEmbeddingsRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createEmbeddings",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=[
                "400",
                "401",
                "402",
                "404",
                "429",
                "4XX",
                "500",
                "502",
                "503",
                "524",
                "529",
                "5XX",
            ],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(
                operations.CreateEmbeddingsResponseBody, http_res
            )
        if utils.match_response(http_res, "200", "text/event-stream"):
            return http_res.text
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "402", "application/json"):
            response_data = unmarshal_json_response(
                errors.PaymentRequiredResponseErrorData, http_res
            )
            raise errors.PaymentRequiredResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "502", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadGatewayResponseErrorData, http_res
            )
            raise errors.BadGatewayResponseError(response_data, http_res)
        if utils.match_response(http_res, "503", "application/json"):
            response_data = unmarshal_json_response(
                errors.ServiceUnavailableResponseErrorData, http_res
            )
            raise errors.ServiceUnavailableResponseError(response_data, http_res)
        if utils.match_response(http_res, "524", "application/json"):
            response_data = unmarshal_json_response(
                errors.EdgeNetworkTimeoutResponseErrorData, http_res
            )
            raise errors.EdgeNetworkTimeoutResponseError(response_data, http_res)
        if utils.match_response(http_res, "529", "application/json"):
            response_data = unmarshal_json_response(
                errors.ProviderOverloadedResponseErrorData, http_res
            )
            raise errors.ProviderOverloadedResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def list_models(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[components.Model]:
        r"""List all embeddings models

        Returns a list of all available embeddings models and their properties

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/embeddings/models",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listEmbeddingsModels",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(components.ModelsListResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def list_models_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[components.Model]:
        r"""List all embeddings models

        Returns a list of all available embeddings models and their properties

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/embeddings/models",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="listEmbeddingsModels",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(components.ModelsListResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .completionlogprobs import CompletionLogprobs, CompletionLogprobsTypedDict
from openrouter.types import BaseModel, Nullable, UNSET_SENTINEL, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Literal, Union
from typing_extensions import Annotated, TypedDict


CompletionFinishReason = Union[
    Literal[
        "stop",
        "length",
        "content_filter",
    ],
    UnrecognizedStr,
]


class CompletionChoiceTypedDict(TypedDict):
    text: str
    index: float
    logprobs: Nullable[CompletionLogprobsTypedDict]
    finish_reason: Nullable[CompletionFinishReason]


class CompletionChoice(BaseModel):
    text: str

    index: float

    logprobs: Nullable[CompletionLogprobs]

    finish_reason: Annotated[
        Nullable[CompletionFinishReason], PlainValidator(validate_open_enum(False))
    ]

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["logprobs", "finish_reason"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from datetime import datetime
from openrouter import components, errors, operations, utils
from openrouter._hooks import HookContext
from openrouter.types import OptionalNullable, UNSET
from openrouter.utils import get_security_from_env
from openrouter.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, List, Mapping, Optional


class APIKeys(BaseSDK):
    r"""API key management endpoints"""

    def list(
        self,
        *,
        include_disabled: Optional[str] = None,
        offset: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[operations.ListData]:
        r"""List API keys

        :param include_disabled: Whether to include disabled API keys in the response
        :param offset: Number of API keys to skip for pagination
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListRequest(
            include_disabled=include_disabled,
            offset=offset,
        )

        req = self._build_request(
            method="GET",
            path="/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.ListResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        include_disabled: Optional[str] = None,
        offset: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[operations.ListData]:
        r"""List API keys

        :param include_disabled: Whether to include disabled API keys in the response
        :param offset: Number of API keys to skip for pagination
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.ListRequest(
            include_disabled=include_disabled,
            offset=offset,
        )

        req = self._build_request_async(
            method="GET",
            path="/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.ListResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def create(
        self,
        *,
        name: str,
        limit: OptionalNullable[float] = UNSET,
        limit_reset: OptionalNullable[operations.CreateKeysLimitReset] = UNSET,
        include_byok_in_limit: Optional[bool] = None,
        expires_at: OptionalNullable[datetime] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateKeysData:
        r"""Create a new API key

        :param name: Name for the new API key
        :param limit: Optional spending limit for the API key in USD
        :param limit_reset: Type of limit reset for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday.
        :param include_byok_in_limit: Whether to include BYOK usage in the limit
        :param expires_at: Optional ISO 8601 UTC timestamp when the API key should expire. Must be UTC, other timezones will be rejected
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateKeysRequest(
            name=name,
            limit=limit,
            limit_reset=limit_reset,
            include_byok_in_limit=include_byok_in_limit,
            expires_at=expires_at,
        )

        req = self._build_request(
            method="POST",
            path="/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateKeysRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(operations.CreateKeysResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        name: str,
        limit: OptionalNullable[float] = UNSET,
        limit_reset: OptionalNullable[operations.CreateKeysLimitReset] = UNSET,
        include_byok_in_limit: Optional[bool] = None,
        expires_at: OptionalNullable[datetime] = UNSET,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.CreateKeysData:
        r"""Create a new API key

        :param name: Name for the new API key
        :param limit: Optional spending limit for the API key in USD
        :param limit_reset: Type of limit reset for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday.
        :param include_byok_in_limit: Whether to include BYOK usage in the limit
        :param expires_at: Optional ISO 8601 UTC timestamp when the API key should expire. Must be UTC, other timezones will be rejected
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.CreateKeysRequest(
            name=name,
            limit=limit,
            limit_reset=limit_reset,
            include_byok_in_limit=include_byok_in_limit,
            expires_at=expires_at,
        )

        req = self._build_request_async(
            method="POST",
            path="/keys",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request, False, False, "json", operations.CreateKeysRequest
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="createKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/json"):
            return unmarshal_json_response(operations.CreateKeysResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def update(
        self,
        *,
        hash: str,
        name: Optional[str] = None,
        disabled: Optional[bool] = None,
        limit: OptionalNullable[float] = UNSET,
        limit_reset: OptionalNullable[operations.UpdateKeysLimitReset] = UNSET,
        include_byok_in_limit: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateKeysData:
        r"""Update an API key

        :param hash: The hash identifier of the API key to update
        :param name: New name for the API key
        :param disabled: Whether to disable the API key
        :param limit: New spending limit for the API key in USD
        :param limit_reset: New limit reset type for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday.
        :param include_byok_in_limit: Whether to include BYOK usage in the limit
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateKeysRequest(
            hash=hash,
            body=operations.UpdateKeysRequestBody(
                name=name,
                disabled=disabled,
                limit=limit,
                limit_reset=limit_reset,
                include_byok_in_limit=include_byok_in_limit,
            ),
        )

        req = self._build_request(
            method="PATCH",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", operations.UpdateKeysRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.UpdateKeysResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def update_async(
        self,
        *,
        hash: str,
        name: Optional[str] = None,
        disabled: Optional[bool] = None,
        limit: OptionalNullable[float] = UNSET,
        limit_reset: OptionalNullable[operations.UpdateKeysLimitReset] = UNSET,
        include_byok_in_limit: Optional[bool] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.UpdateKeysData:
        r"""Update an API key

        :param hash: The hash identifier of the API key to update
        :param name: New name for the API key
        :param disabled: Whether to disable the API key
        :param limit: New spending limit for the API key in USD
        :param limit_reset: New limit reset type for the API key (daily, weekly, monthly, or null for no reset). Resets happen automatically at midnight UTC, and weeks are Monday through Sunday.
        :param include_byok_in_limit: Whether to include BYOK usage in the limit
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.UpdateKeysRequest(
            hash=hash,
            body=operations.UpdateKeysRequestBody(
                name=name,
                disabled=disabled,
                limit=limit,
                limit_reset=limit_reset,
                include_byok_in_limit=include_byok_in_limit,
            ),
        )

        req = self._build_request_async(
            method="PATCH",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.body, False, False, "json", operations.UpdateKeysRequestBody
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="updateKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.UpdateKeysResponse, http_res)
        if utils.match_response(http_res, "400", "application/json"):
            response_data = unmarshal_json_response(
                errors.BadRequestResponseErrorData, http_res
            )
            raise errors.BadRequestResponseError(response_data, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def delete(
        self,
        *,
        hash: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DeleteKeysResponse:
        r"""Delete an API key

        :param hash: The hash identifier of the API key to delete
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DeleteKeysRequest(
            hash=hash,
        )

        req = self._build_request(
            method="DELETE",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.DeleteKeysResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def delete_async(
        self,
        *,
        hash: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.DeleteKeysResponse:
        r"""Delete an API key

        :param hash: The hash identifier of the API key to delete
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.DeleteKeysRequest(
            hash=hash,
        )

        req = self._build_request_async(
            method="DELETE",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="deleteKeys",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.DeleteKeysResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def get(
        self,
        *,
        hash: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetKeyData:
        r"""Get a single API key

        :param hash: The hash identifier of the API key to retrieve
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetKeyRequest(
            hash=hash,
        )

        req = self._build_request(
            method="GET",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetKeyResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        hash: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetKeyData:
        r"""Get a single API key

        :param hash: The hash identifier of the API key to retrieve
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = operations.GetKeyRequest(
            hash=hash,
        )

        req = self._build_request_async(
            method="GET",
            path="/keys/{hash}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "404", "429", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetKeyResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "404", "application/json"):
            response_data = unmarshal_json_response(
                errors.NotFoundResponseErrorData, http_res
            )
            raise errors.NotFoundResponseError(response_data, http_res)
        if utils.match_response(http_res, "429", "application/json"):
            response_data = unmarshal_json_response(
                errors.TooManyRequestsResponseErrorData, http_res
            )
            raise errors.TooManyRequestsResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    def get_current_key_metadata(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetCurrentKeyData:
        r"""Get current API key

        Get information on the API key associated with the current authentication session

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request(
            method="GET",
            path="/key",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCurrentKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetCurrentKeyResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)

    async def get_current_key_metadata_async(
        self,
        *,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> operations.GetCurrentKeyData:
        r"""Get current API key

        Get information on the API key associated with the current authentication session

        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)
        req = self._build_request_async(
            method="GET",
            path="/key",
            base_url=base_url,
            url_variables=url_variables,
            request=None,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="getCurrentKey",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, components.Security
                ),
            ),
            request=req,
            error_status_codes=["401", "4XX", "500", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return unmarshal_json_response(operations.GetCurrentKeyResponse, http_res)
        if utils.match_response(http_res, "401", "application/json"):
            response_data = unmarshal_json_response(
                errors.UnauthorizedResponseErrorData, http_res
            )
            raise errors.UnauthorizedResponseError(response_data, http_res)
        if utils.match_response(http_res, "500", "application/json"):
            response_data = unmarshal_json_response(
                errors.InternalServerResponseErrorData, http_res
            )
            raise errors.InternalServerResponseError(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise errors.OpenRouterDefaultError(
                "API error occurred", http_res, http_res_text
            )

        raise errors.OpenRouterDefaultError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .inputmodality import InputModality
from .instructtype import InstructType
from .outputmodality import OutputModality
from .publicendpoint import PublicEndpoint, PublicEndpointTypedDict
from openrouter.types import BaseModel, Nullable, UNSET_SENTINEL, UnrecognizedStr
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import List, Literal, Union
from typing_extensions import Annotated, TypedDict


Tokenizer = Union[
    Literal[
        "Router",
        "Media",
        "Other",
        "GPT",
        "Claude",
        "Gemini",
        "Grok",
        "Cohere",
        "Nova",
        "Qwen",
        "Yi",
        "DeepSeek",
        "Mistral",
        "Llama2",
        "Llama3",
        "Llama4",
        "PaLM",
        "RWKV",
        "Qwen3",
    ],
    UnrecognizedStr,
]
r"""Tokenizer type used by the model"""


class ArchitectureTypedDict(TypedDict):
    r"""Model architecture information"""

    tokenizer: Nullable[Tokenizer]
    instruct_type: Nullable[InstructType]
    r"""Instruction format type"""
    modality: Nullable[str]
    r"""Primary modality of the model"""
    input_modalities: List[InputModality]
    r"""Supported input modalities"""
    output_modalities: List[OutputModality]
    r"""Supported output modalities"""


class Architecture(BaseModel):
    r"""Model architecture information"""

    tokenizer: Annotated[Nullable[Tokenizer], PlainValidator(validate_open_enum(False))]

    instruct_type: Annotated[
        Nullable[InstructType], PlainValidator(validate_open_enum(False))
    ]
    r"""Instruction format type"""

    modality: Nullable[str]
    r"""Primary modality of the model"""

    input_modalities: List[
        Annotated[InputModality, PlainValidator(validate_open_enum(False))]
    ]
    r"""Supported input modalities"""

    output_modalities: List[
        Annotated[OutputModality, PlainValidator(validate_open_enum(False))]
    ]
    r"""Supported output modalities"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["tokenizer", "instruct_type", "modality"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class ListEndpointsResponseTypedDict(TypedDict):
    r"""List of available endpoints for a model"""

    id: str
    r"""Unique identifier for the model"""
    name: str
    r"""Display name of the model"""
    created: float
    r"""Unix timestamp of when the model was created"""
    description: str
    r"""Description of the model"""
    architecture: ArchitectureTypedDict
    endpoints: List[PublicEndpointTypedDict]
    r"""List of available endpoints for this model"""


class ListEndpointsResponse(BaseModel):
    r"""List of available endpoints for a model"""

    id: str
    r"""Unique identifier for the model"""

    name: str
    r"""Display name of the model"""

    created: float
    r"""Unix timestamp of when the model was created"""

    description: str
    r"""Description of the model"""

    architecture: Architecture

    endpoints: List[PublicEndpoint]
    r"""List of available endpoints for this model"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitemaudio import (
    ChatMessageContentItemAudio,
    ChatMessageContentItemAudioTypedDict,
)
from .chatmessagecontentitemimage import (
    ChatMessageContentItemImage,
    ChatMessageContentItemImageTypedDict,
)
from .chatmessagecontentitemtext import (
    ChatMessageContentItemText,
    ChatMessageContentItemTextTypedDict,
)
from .chatmessagecontentitemvideo import (
    ChatMessageContentItemVideo,
    ChatMessageContentItemVideoTypedDict,
)
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import Union
from typing_extensions import Annotated, TypeAliasType


ChatMessageContentItemTypedDict = TypeAliasType(
    "ChatMessageContentItemTypedDict",
    Union[
        ChatMessageContentItemTextTypedDict,
        ChatMessageContentItemImageTypedDict,
        ChatMessageContentItemAudioTypedDict,
        ChatMessageContentItemVideoTypedDict,
    ],
)


ChatMessageContentItem = Annotated[
    Union[
        Annotated[ChatMessageContentItemText, Tag("text")],
        Annotated[ChatMessageContentItemImage, Tag("image_url")],
        Annotated[ChatMessageContentItemAudio, Tag("input_audio")],
        Annotated[ChatMessageContentItemVideo, Tag("input_video")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .openairesponsesincompletedetails import (
    OpenAIResponsesIncompleteDetails,
    OpenAIResponsesIncompleteDetailsTypedDict,
)
from .openairesponsesinput_union import (
    OpenAIResponsesInputUnion,
    OpenAIResponsesInputUnionTypedDict,
)
from .openairesponsesprompt import OpenAIResponsesPrompt, OpenAIResponsesPromptTypedDict
from .openairesponsesreasoningconfig import (
    OpenAIResponsesReasoningConfig,
    OpenAIResponsesReasoningConfigTypedDict,
)
from .openairesponsesresponsestatus import OpenAIResponsesResponseStatus
from .openairesponsesservicetier import OpenAIResponsesServiceTier
from .openairesponsestoolchoice_union import (
    OpenAIResponsesToolChoiceUnion,
    OpenAIResponsesToolChoiceUnionTypedDict,
)
from .openairesponsestruncation import OpenAIResponsesTruncation
from .openresponsesusage import OpenResponsesUsage, OpenResponsesUsageTypedDict
from .openresponseswebsearch20250826tool import (
    OpenResponsesWebSearch20250826Tool,
    OpenResponsesWebSearch20250826ToolTypedDict,
)
from .openresponseswebsearchpreview20250311tool import (
    OpenResponsesWebSearchPreview20250311Tool,
    OpenResponsesWebSearchPreview20250311ToolTypedDict,
)
from .openresponseswebsearchpreviewtool import (
    OpenResponsesWebSearchPreviewTool,
    OpenResponsesWebSearchPreviewToolTypedDict,
)
from .openresponseswebsearchtool import (
    OpenResponsesWebSearchTool,
    OpenResponsesWebSearchToolTypedDict,
)
from .responseserrorfield import ResponsesErrorField, ResponsesErrorFieldTypedDict
from .responsesoutputitem import ResponsesOutputItem, ResponsesOutputItemTypedDict
from .responsetextconfig import ResponseTextConfig, ResponseTextConfigTypedDict
from openrouter.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from openrouter.utils import validate_open_enum
from pydantic import model_serializer
from pydantic.functional_validators import PlainValidator
from typing import Any, Dict, List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


Object = Literal["response",]


OpenResponsesNonStreamingResponseType = Literal["function",]


class OpenResponsesNonStreamingResponseToolFunctionTypedDict(TypedDict):
    r"""Function tool definition"""

    type: OpenResponsesNonStreamingResponseType
    name: str
    parameters: Nullable[Dict[str, Nullable[Any]]]
    description: NotRequired[Nullable[str]]
    strict: NotRequired[Nullable[bool]]


class OpenResponsesNonStreamingResponseToolFunction(BaseModel):
    r"""Function tool definition"""

    type: OpenResponsesNonStreamingResponseType

    name: str

    parameters: Nullable[Dict[str, Nullable[Any]]]

    description: OptionalNullable[str] = UNSET

    strict: OptionalNullable[bool] = UNSET

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = ["description", "strict"]
        nullable_fields = ["description", "strict", "parameters"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


OpenResponsesNonStreamingResponseToolUnionTypedDict = TypeAliasType(
    "OpenResponsesNonStreamingResponseToolUnionTypedDict",
    Union[
        OpenResponsesWebSearchPreviewToolTypedDict,
        OpenResponsesWebSearchPreview20250311ToolTypedDict,
        OpenResponsesWebSearchToolTypedDict,
        OpenResponsesWebSearch20250826ToolTypedDict,
        OpenResponsesNonStreamingResponseToolFunctionTypedDict,
    ],
)


OpenResponsesNonStreamingResponseToolUnion = TypeAliasType(
    "OpenResponsesNonStreamingResponseToolUnion",
    Union[
        OpenResponsesWebSearchPreviewTool,
        OpenResponsesWebSearchPreview20250311Tool,
        OpenResponsesWebSearchTool,
        OpenResponsesWebSearch20250826Tool,
        OpenResponsesNonStreamingResponseToolFunction,
    ],
)


class OpenResponsesNonStreamingResponseTypedDict(TypedDict):
    r"""Complete non-streaming response from the Responses API"""

    id: str
    object: Object
    created_at: float
    model: str
    output: List[ResponsesOutputItemTypedDict]
    error: Nullable[ResponsesErrorFieldTypedDict]
    r"""Error information returned from the API"""
    incomplete_details: Nullable[OpenAIResponsesIncompleteDetailsTypedDict]
    temperature: Nullable[float]
    top_p: Nullable[float]
    instructions: Nullable[OpenAIResponsesInputUnionTypedDict]
    metadata: Nullable[Dict[str, str]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""
    tools: List[OpenResponsesNonStreamingResponseToolUnionTypedDict]
    tool_choice: OpenAIResponsesToolChoiceUnionTypedDict
    parallel_tool_calls: bool
    status: NotRequired[OpenAIResponsesResponseStatus]
    user: NotRequired[Nullable[str]]
    output_text: NotRequired[str]
    prompt_cache_key: NotRequired[Nullable[str]]
    safety_identifier: NotRequired[Nullable[str]]
    usage: NotRequired[OpenResponsesUsageTypedDict]
    r"""Token usage information for the response"""
    max_tool_calls: NotRequired[Nullable[float]]
    top_logprobs: NotRequired[float]
    max_output_tokens: NotRequired[Nullable[float]]
    prompt: NotRequired[Nullable[OpenAIResponsesPromptTypedDict]]
    background: NotRequired[Nullable[bool]]
    previous_response_id: NotRequired[Nullable[str]]
    reasoning: NotRequired[Nullable[OpenAIResponsesReasoningConfigTypedDict]]
    service_tier: NotRequired[Nullable[OpenAIResponsesServiceTier]]
    store: NotRequired[bool]
    truncation: NotRequired[Nullable[OpenAIResponsesTruncation]]
    text: NotRequired[ResponseTextConfigTypedDict]
    r"""Text output configuration including format and verbosity"""


class OpenResponsesNonStreamingResponse(BaseModel):
    r"""Complete non-streaming response from the Responses API"""

    id: str

    object: Object

    created_at: float

    model: str

    output: List[ResponsesOutputItem]

    error: Nullable[ResponsesErrorField]
    r"""Error information returned from the API"""

    incomplete_details: Nullable[OpenAIResponsesIncompleteDetails]

    temperature: Nullable[float]

    top_p: Nullable[float]

    instructions: Nullable[OpenAIResponsesInputUnion]

    metadata: Nullable[Dict[str, str]]
    r"""Metadata key-value pairs for the request. Keys must be ≤64 characters and cannot contain brackets. Values must be ≤512 characters. Maximum 16 pairs allowed."""

    tools: List[OpenResponsesNonStreamingResponseToolUnion]

    tool_choice: OpenAIResponsesToolChoiceUnion

    parallel_tool_calls: bool

    status: Annotated[
        Optional[OpenAIResponsesResponseStatus],
        PlainValidator(validate_open_enum(False)),
    ] = None

    user: OptionalNullable[str] = UNSET

    output_text: Optional[str] = None

    prompt_cache_key: OptionalNullable[str] = UNSET

    safety_identifier: OptionalNullable[str] = UNSET

    usage: Optional[OpenResponsesUsage] = None
    r"""Token usage information for the response"""

    max_tool_calls: OptionalNullable[float] = UNSET

    top_logprobs: Optional[float] = None

    max_output_tokens: OptionalNullable[float] = UNSET

    prompt: OptionalNullable[OpenAIResponsesPrompt] = UNSET

    background: OptionalNullable[bool] = UNSET

    previous_response_id: OptionalNullable[str] = UNSET

    reasoning: OptionalNullable[OpenAIResponsesReasoningConfig] = UNSET

    service_tier: Annotated[
        OptionalNullable[OpenAIResponsesServiceTier],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    store: Optional[bool] = None

    truncation: Annotated[
        OptionalNullable[OpenAIResponsesTruncation],
        PlainValidator(validate_open_enum(False)),
    ] = UNSET

    text: Optional[ResponseTextConfig] = None
    r"""Text output configuration including format and verbosity"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "status",
            "user",
            "output_text",
            "prompt_cache_key",
            "safety_identifier",
            "usage",
            "max_tool_calls",
            "top_logprobs",
            "max_output_tokens",
            "prompt",
            "background",
            "previous_response_id",
            "reasoning",
            "service_tier",
            "store",
            "truncation",
            "text",
        ]
        nullable_fields = [
            "user",
            "prompt_cache_key",
            "safety_identifier",
            "error",
            "incomplete_details",
            "max_tool_calls",
            "max_output_tokens",
            "temperature",
            "top_p",
            "instructions",
            "metadata",
            "prompt",
            "background",
            "previous_response_id",
            "reasoning",
            "service_tier",
            "truncation",
        ]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responseinputaudio import ResponseInputAudio, ResponseInputAudioTypedDict
from .responseinputfile import ResponseInputFile, ResponseInputFileTypedDict
from .responseinputimage import ResponseInputImage, ResponseInputImageTypedDict
from .responseinputtext import ResponseInputText, ResponseInputTextTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesInputMessageItemType = Literal["message",]


OpenResponsesInputMessageItemRoleDeveloper = Literal["developer",]


OpenResponsesInputMessageItemRoleSystem = Literal["system",]


OpenResponsesInputMessageItemRoleUser = Literal["user",]


OpenResponsesInputMessageItemRoleUnionTypedDict = TypeAliasType(
    "OpenResponsesInputMessageItemRoleUnionTypedDict",
    Union[
        OpenResponsesInputMessageItemRoleUser,
        OpenResponsesInputMessageItemRoleSystem,
        OpenResponsesInputMessageItemRoleDeveloper,
    ],
)


OpenResponsesInputMessageItemRoleUnion = TypeAliasType(
    "OpenResponsesInputMessageItemRoleUnion",
    Union[
        OpenResponsesInputMessageItemRoleUser,
        OpenResponsesInputMessageItemRoleSystem,
        OpenResponsesInputMessageItemRoleDeveloper,
    ],
)


OpenResponsesInputMessageItemContentTypedDict = TypeAliasType(
    "OpenResponsesInputMessageItemContentTypedDict",
    Union[
        ResponseInputTextTypedDict,
        ResponseInputAudioTypedDict,
        ResponseInputImageTypedDict,
        ResponseInputFileTypedDict,
    ],
)


OpenResponsesInputMessageItemContent = Annotated[
    Union[
        Annotated[ResponseInputText, Tag("input_text")],
        Annotated[ResponseInputImage, Tag("input_image")],
        Annotated[ResponseInputFile, Tag("input_file")],
        Annotated[ResponseInputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


class OpenResponsesInputMessageItemTypedDict(TypedDict):
    role: OpenResponsesInputMessageItemRoleUnionTypedDict
    content: List[OpenResponsesInputMessageItemContentTypedDict]
    id: NotRequired[str]
    type: NotRequired[OpenResponsesInputMessageItemType]


class OpenResponsesInputMessageItem(BaseModel):
    role: OpenResponsesInputMessageItemRoleUnion

    content: List[OpenResponsesInputMessageItemContent]

    id: Optional[str] = None

    type: Optional[OpenResponsesInputMessageItemType] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .responseinputaudio import ResponseInputAudio, ResponseInputAudioTypedDict
from .responseinputfile import ResponseInputFile, ResponseInputFileTypedDict
from .responseinputimage import ResponseInputImage, ResponseInputImageTypedDict
from .responseinputtext import ResponseInputText, ResponseInputTextTypedDict
from openrouter.types import BaseModel
from openrouter.utils import get_discriminator
from pydantic import Discriminator, Tag
from typing import List, Literal, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


OpenResponsesEasyInputMessageType = Literal["message",]


OpenResponsesEasyInputMessageRoleDeveloper = Literal["developer",]


OpenResponsesEasyInputMessageRoleAssistant = Literal["assistant",]


OpenResponsesEasyInputMessageRoleSystem = Literal["system",]


OpenResponsesEasyInputMessageRoleUser = Literal["user",]


OpenResponsesEasyInputMessageRoleUnionTypedDict = TypeAliasType(
    "OpenResponsesEasyInputMessageRoleUnionTypedDict",
    Union[
        OpenResponsesEasyInputMessageRoleUser,
        OpenResponsesEasyInputMessageRoleSystem,
        OpenResponsesEasyInputMessageRoleAssistant,
        OpenResponsesEasyInputMessageRoleDeveloper,
    ],
)


OpenResponsesEasyInputMessageRoleUnion = TypeAliasType(
    "OpenResponsesEasyInputMessageRoleUnion",
    Union[
        OpenResponsesEasyInputMessageRoleUser,
        OpenResponsesEasyInputMessageRoleSystem,
        OpenResponsesEasyInputMessageRoleAssistant,
        OpenResponsesEasyInputMessageRoleDeveloper,
    ],
)


OpenResponsesEasyInputMessageContent1TypedDict = TypeAliasType(
    "OpenResponsesEasyInputMessageContent1TypedDict",
    Union[
        ResponseInputTextTypedDict,
        ResponseInputAudioTypedDict,
        ResponseInputImageTypedDict,
        ResponseInputFileTypedDict,
    ],
)


OpenResponsesEasyInputMessageContent1 = Annotated[
    Union[
        Annotated[ResponseInputText, Tag("input_text")],
        Annotated[ResponseInputImage, Tag("input_image")],
        Annotated[ResponseInputFile, Tag("input_file")],
        Annotated[ResponseInputAudio, Tag("input_audio")],
    ],
    Discriminator(lambda m: get_discriminator(m, "type", "type")),
]


OpenResponsesEasyInputMessageContent2TypedDict = TypeAliasType(
    "OpenResponsesEasyInputMessageContent2TypedDict",
    Union[List[OpenResponsesEasyInputMessageContent1TypedDict], str],
)


OpenResponsesEasyInputMessageContent2 = TypeAliasType(
    "OpenResponsesEasyInputMessageContent2",
    Union[List[OpenResponsesEasyInputMessageContent1], str],
)


class OpenResponsesEasyInputMessageTypedDict(TypedDict):
    role: OpenResponsesEasyInputMessageRoleUnionTypedDict
    content: OpenResponsesEasyInputMessageContent2TypedDict
    type: NotRequired[OpenResponsesEasyInputMessageType]


class OpenResponsesEasyInputMessage(BaseModel):
    role: OpenResponsesEasyInputMessageRoleUnion

    content: OpenResponsesEasyInputMessageContent2

    type: Optional[OpenResponsesEasyInputMessageType] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .chatmessagecontentitem import (
    ChatMessageContentItem,
    ChatMessageContentItemTypedDict,
)
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import List, Literal, Union
from typing_extensions import Annotated, TypeAliasType, TypedDict


ToolResponseMessageContentTypedDict = TypeAliasType(
    "ToolResponseMessageContentTypedDict",
    Union[str, List[ChatMessageContentItemTypedDict]],
)


ToolResponseMessageContent = TypeAliasType(
    "ToolResponseMessageContent", Union[str, List[ChatMessageContentItem]]
)


class ToolResponseMessageTypedDict(TypedDict):
    content: ToolResponseMessageContentTypedDict
    tool_call_id: str
    role: Literal["tool"]


class ToolResponseMessage(BaseModel):
    content: ToolResponseMessageContent

    tool_call_id: str

    ROLE: Annotated[
        Annotated[Literal["tool"], AfterValidator(validate_const("tool"))],
        pydantic.Field(alias="role"),
    ] = "tool"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing import Literal
from typing_extensions import TypedDict


OpenResponsesImageGenCallInProgressType = Literal[
    "response.image_generation_call.in_progress",
]


class OpenResponsesImageGenCallInProgressTypedDict(TypedDict):
    r"""Image generation call in progress"""

    type: OpenResponsesImageGenCallInProgressType
    item_id: str
    output_index: float
    sequence_number: float


class OpenResponsesImageGenCallInProgress(BaseModel):
    r"""Image generation call in progress"""

    type: OpenResponsesImageGenCallInProgressType

    item_id: str

    output_index: float

    sequence_number: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class ActivityItemTypedDict(TypedDict):
    date_: str
    r"""Date of the activity (YYYY-MM-DD format)"""
    model: str
    r"""Model slug (e.g., \"openai/gpt-4.1\")"""
    model_permaslug: str
    r"""Model permaslug (e.g., \"openai/gpt-4.1-2025-04-14\")"""
    endpoint_id: str
    r"""Unique identifier for the endpoint"""
    provider_name: str
    r"""Name of the provider serving this endpoint"""
    usage: float
    r"""Total cost in USD (OpenRouter credits spent)"""
    byok_usage_inference: float
    r"""BYOK inference cost in USD (external credits spent)"""
    requests: float
    r"""Number of requests made"""
    prompt_tokens: float
    r"""Total prompt tokens used"""
    completion_tokens: float
    r"""Total completion tokens generated"""
    reasoning_tokens: float
    r"""Total reasoning tokens used"""


class ActivityItem(BaseModel):
    date_: Annotated[str, pydantic.Field(alias="date")]
    r"""Date of the activity (YYYY-MM-DD format)"""

    model: str
    r"""Model slug (e.g., \"openai/gpt-4.1\")"""

    model_permaslug: str
    r"""Model permaslug (e.g., \"openai/gpt-4.1-2025-04-14\")"""

    endpoint_id: str
    r"""Unique identifier for the endpoint"""

    provider_name: str
    r"""Name of the provider serving this endpoint"""

    usage: float
    r"""Total cost in USD (OpenRouter credits spent)"""

    byok_usage_inference: float
    r"""BYOK inference cost in USD (external credits spent)"""

    requests: float
    r"""Number of requests made"""

    prompt_tokens: float
    r"""Total prompt tokens used"""

    completion_tokens: float
    r"""Total completion tokens generated"""

    reasoning_tokens: float
    r"""Total reasoning tokens used"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from typing_extensions import TypedDict


class CompletionUsageTypedDict(TypedDict):
    prompt_tokens: float
    completion_tokens: float
    total_tokens: float


class CompletionUsage(BaseModel):
    prompt_tokens: float

    completion_tokens: float

    total_tokens: float



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import BaseModel
from openrouter.utils import validate_const
import pydantic
from pydantic.functional_validators import AfterValidator
from typing import Literal
from typing_extensions import Annotated, TypedDict


class ChatMessageToolCallFunctionTypedDict(TypedDict):
    name: str
    arguments: str


class ChatMessageToolCallFunction(BaseModel):
    name: str

    arguments: str


class ChatMessageToolCallTypedDict(TypedDict):
    id: str
    function: ChatMessageToolCallFunctionTypedDict
    type: Literal["function"]


class ChatMessageToolCall(BaseModel):
    id: str

    function: ChatMessageToolCallFunction

    TYPE: Annotated[
        Annotated[Literal["function"], AfterValidator(validate_const("function"))],
        pydantic.Field(alias="type"),
    ] = "function"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from openrouter.types import UnrecognizedStr
from typing import Literal, Union


OpenAIResponsesResponseStatus = Union[
    Literal[
        "completed",
        "incomplete",
        "in_progress",
        "failed",
        "cancelled",
        "queued",
    ],
    UnrecognizedStr,
]
